import { QueryBusCallback, QueryHandler } from './bus.js';
import Query, { QueryClass } from './query.js';
import QueryHandlers from './query-handlers.js';
import '@hexadrop/either';
import '@hexadrop/error';
import '@hexadrop/types/class';
import '@hexadrop/types/nullable';
import '@hexadrop/types/primitives';

/**
 * This is a class named InMemoryQueryHandlers that extends the abstract class QueryHandlers.
 * It provides an in-memory implementation for handling queries in the application.
 */
declare class InMemoryQueryHandlers extends QueryHandlers {
    /**
     * A private readonly property named 'queryHandlersMap'.
     * It is a Map where the keys are strings (query names) and the values are QueryBusCallback functions.
     */
    private readonly queryHandlersMap;
    /**
     * This is the constructor method for the class.
     * It initializes the 'queryHandlersMap' property as a new Map.
     */
    constructor();
    /**
     * This is a public method named 'register'.
     * It takes a query and a handler or callback as arguments.
     * The method registers the handler or callback to the 'queryHandlersMap' with the query name as the key.
     *
     * @param {QueryClass<ResponseType, QueryType>} query - The query to be registered.
     * @param {QueryBusCallback<ResponseType, QueryType> | QueryHandler<ResponseType, QueryType>} handlerOrCallback - The handler or callback to be registered.
     */
    register<ResponseType, QueryType extends Query<ResponseType>>(query: QueryClass<ResponseType, QueryType>, handlerOrCallback: QueryBusCallback<ResponseType, QueryType> | QueryHandler<ResponseType, QueryType>): void;
    /**
     * This is a public method named 'search'.
     * It takes a query as an argument.
     * The method searches the 'queryHandlersMap' for the handler or callback registered with the query name.
     * If the query name is not found, it throws an 'InvalidQueryError'.
     * If the handler or callback is not found, it throws a 'QueryNotRegisteredError'.
     *
     * @param {QueryType | QueryClass<ResponseType, QueryType>} query - The query to be searched.
     * @returns {QueryBusCallback<ResponseType, QueryType>} - The handler or callback registered with the query name.
     */
    search<ResponseType, QueryType extends Query<ResponseType>>(query: QueryType | QueryClass<ResponseType, QueryType>): QueryBusCallback<ResponseType, QueryType>;
    /**
     * This is a public method named 'unregister'.
     * It takes a query as an argument.
     * The method removes the handler or callback registered with the query name from the 'queryHandlersMap'.
     *
     * @param {QueryClass<unknown, Query>} query - The query to be unregistered.
     */
    unregister(query: QueryClass<unknown, Query>): void;
}

export { InMemoryQueryHandlers as default };
